#include "stdafx.h"
#include "ContrastEnhancementDlg.h"
#include "ContrastEnhancement.h"

#include "resource.h"

// Ignore some stuff from commctrl.h
#define    NOTOOLBAR    Customizable bitmap-button toolbar control.
#define    NOUPDOWN     Up and Down arrow increment/decrement control.
#define    NOSTATUSBAR  Status bar control.
#define    NOMENUHELP   APIs to help manage menus, especially with a status bar.
//#define    NOTRACKBAR   Customizable column-width tracking control.
#define    NODRAGLIST   APIs to make a listbox source and sink drag&drop actions.
#define    NOPROGRESS   Progress gas gauge.
#define    NOHOTKEY     HotKey control
#define    NOHEADER     Header bar control.
#define    NOIMAGEAPIS  ImageList apis.
#define    NOLISTVIEW   ListView control.
#define    NOTREEVIEW   TreeView control.
#define    NOTABCONTROL Tab control.
#define    NOANIMATE    Animate control

#include <commctrl.h>
#include <commdlg.h>

#ifdef FILTERDLL_EXPORTS
extern HANDLE g_hModule;
#endif


//////////////////////////////////////////////////////////////////////////
// Dialog Message Handler
//////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK CContrastEnhancementDlgMessage( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	if( message == WM_INITDIALOG )
	{
		::SetWindowLong( hDlg, (-21), (long)lParam );
	}
	((CContrastEnhancementDlg*)::GetWindowLong( hDlg, (-21)) )->DlgMessage( hDlg, message, wParam, lParam );

	return FALSE; 
}


//////////////////////////////////////////////////////////////////////////

CContrastEnhancementDlg::CContrastEnhancementDlg( CContrastEnhancement* pCContrastEnhancement )
{
	m_pCContrastEnhancement = pCContrastEnhancement;
}

void CContrastEnhancementDlg::show(HWND hParent)
{ 	
#ifdef FILTERDLL_EXPORTS
	DialogBoxParam( (HINSTANCE)g_hModule, (LPCTSTR)IDD_DIALOG_CCONTRASTENHANCEMENT, hParent, 
		(DLGPROC)CContrastEnhancementDlgMessage, (long)this );
#else
	DialogBoxParam( AfxGetInstanceHandle(), (LPCTSTR)IDD_DIALOG_CCONTRASTENHANCEMENT, hParent, 
		(DLGPROC)CContrastEnhancementDlgMessage, (long)this );
#endif
}


//////////////////////////////////////////////////////////////////////////
// The message handler for the dialog
//////////////////////////////////////////////////////////////////////////

LRESULT CContrastEnhancementDlg::DlgMessage( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
		case WM_INITDIALOG:
			CreateControls( hDlg );
			return TRUE;

		case WM_HSCROLL:
			OnSliderChanged( hDlg, (HWND)lParam );
			break;
		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
			case IDC_CHECKACTIVE:
				OnCheckEnable( hDlg );
				break;

			case IDOK:
			case IDCANCEL:
				::EndDialog( hDlg, LOWORD(wParam) );
				return TRUE;
			}
			break;
	}
	return FALSE;
}


//////////////////////////////////////////////////////////////////////////
// Create the controls of the dialog and initialize them according
// to the appropriate parameter values.
//////////////////////////////////////////////////////////////////////////

void CContrastEnhancementDlg::CreateControls( HWND hDlg )
{
	// Initialize the Enabled check box.
	bool bEnabled;
	HWND hCheckEnabled = ::GetDlgItem( hDlg, IDC_CHECKACTIVE );

	m_pCContrastEnhancement->getEnabled( bEnabled );
	if( bEnabled )
	{
		::SendMessage( hCheckEnabled, BM_SETCHECK, BST_CHECKED, 0 );
	}
	else
	{
		::SendMessage( hCheckEnabled, BM_SETCHECK, BST_UNCHECKED, 0 );
	}

	// Read the current property values from the filter properties.
	long lowerBound;
	m_pCContrastEnhancement->getLowerBound(lowerBound);
	long upperBound;
	m_pCContrastEnhancement->getUpperBound(upperBound);

	// Initialize the sliders
	HWND hLowerBound = ::GetDlgItem( hDlg, IDC_LOWERBOUND );
	HWND hUpperBound = ::GetDlgItem( hDlg, IDC_UPPERBOUND );

	::SendMessage( hLowerBound, TBM_SETRANGE, TRUE, MAKELONG( 0, 255 ) );
	::SendMessage( hUpperBound, TBM_SETRANGE, TRUE, MAKELONG( 0, 255 ) );

	::SendMessage( hLowerBound, TBM_SETPOS, TRUE, lowerBound );
	::SendMessage( hUpperBound, TBM_SETPOS, TRUE, upperBound );

	// Initialize the text fields displaying the slider values. 
	::SetDlgItemInt( hDlg, IDC_LOWERBOUNDLABEL, lowerBound, false );
	::SetDlgItemInt( hDlg, IDC_UPPERBOUNDLABEL, upperBound, false );

	::ShowWindow(hDlg, SW_SHOW);
}



//////////////////////////////////////////////////////////////////////////
// Handle the Enabled check box.
//////////////////////////////////////////////////////////////////////////

void CContrastEnhancementDlg::OnCheckEnable( HWND hDlg )
{
	HWND hCheckEnabled = ::GetDlgItem( hDlg, IDC_CHECKACTIVE );

	// Get the check state of the Enabled check box and set the filter property accordingly.
	if( ::SendMessage( hCheckEnabled, BM_GETCHECK, 0, 0 ) == BST_CHECKED )
	{
		m_pCContrastEnhancement->setEnabled( true );
	}
	else
	{
		m_pCContrastEnhancement->setEnabled( false );
	}	
}


//////////////////////////////////////////////////////////////////////////
// Handle the Lower Bound and Upper Bound slider
//////////////////////////////////////////////////////////////////////////

void CContrastEnhancementDlg::OnSliderChanged( HWND hDlg, HWND hSlider )
{	
	// Get the slider handles
	HWND hLowerBound = ::GetDlgItem( hDlg, IDC_LOWERBOUND );
	HWND hUpperBound = ::GetDlgItem( hDlg, IDC_UPPERBOUND );

	// Read the current slider values.
	long lowerBound = ::SendMessage( hLowerBound, TBM_GETPOS, 0, 0 );
	long upperBound = ::SendMessage( hUpperBound, TBM_GETPOS, 0, 0 );

	// Make sure the lower bound value is smaller than the upper bound value.
	if( hSlider == hLowerBound )
	{
		if( upperBound <= lowerBound ) lowerBound = upperBound - 1;
	}
	else if( hSlider == hUpperBound )
	{
		if( upperBound <= lowerBound ) upperBound = lowerBound + 1;
	}

	// Set the filter properties according to the corrected values.
	m_pCContrastEnhancement->setLowerBound( lowerBound );
	m_pCContrastEnhancement->setUpperBound( upperBound );

	// Write back the corrected values to the sliders.
	::SendMessage( hLowerBound, TBM_SETPOS, TRUE, lowerBound );
	::SendMessage( hUpperBound, TBM_SETPOS, TRUE, upperBound );

	// Update the text fields displaying the slider values.
	::SetDlgItemInt( hDlg, IDC_LOWERBOUNDLABEL, lowerBound, false );
	::SetDlgItemInt( hDlg, IDC_UPPERBOUNDLABEL, upperBound, false );
}
